/*
 *  CsvJdbc - a JDBC driver for CSV files
 *  Copyright (C) 2001  Jonathan Ackerman
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.relique.jdbc.csv;

import java.io.*;
import java.util.*;
import java.sql.SQLException;

/**
 * This class is a helper class that handles the reading and parsing of data
 * from a .csv file.
 *
 * @author     Jonathan Ackerman
 * @author     Sander Brienen
 * @author     Stuart Mottram (fritto)
 * @author     Jason Bedell
 * @created    25 November 2001
 * @version    $Id: CsvReader.java,v 1.9 2002/09/04 17:03:23 mmaraya Exp $
 */

public class CsvWriter
{
  private RandomAccessFile input;
  private RandomAccessFile output;
  private String[] columnNames;
  private String[] columns;
  private java.lang.String buf = null;
  private char separator = ',';
  private String tableName;
  private String fileName;
  private int counter;
  private long current;
  private long endLine;


  /**
   *Constructor for the CsvReader object
   *
   * @param  fileName       Description of Parameter
   * @exception  Exception  Description of Exception
   * @since
   */
  public CsvWriter(String fileName) throws Exception
  {
    this(fileName, ',');
  }


  /**
   * Insert the method's description here.
   *
   * Creation date: (6-11-2001 15:02:42)
   *
   * @param  fileName                 java.lang.String
   * @param  separator                char
   * @exception  java.lang.Exception  The exception description.
   * @since
   */
  public CsvWriter(String fileName, char separator)
       throws java.lang.Exception
  {
    this.separator = separator;
    this.fileName = fileName;
    
    this.input = new RandomAccessFile(fileName,"rw");
    String headerLine = this.input.readLine();
    columnNames = parseCsvLine(headerLine);
    counter = 1;
    current = 0;
    endLine=0;
  }


  /**
   *Gets the columnNames attribute of the CsvReader object
   *
   * @return    The columnNames value
   * @since
   */
  public String[] getColumnNames()
  {
    return columnNames;
  }


  public String getTableName() {
      if(tableName != null)
          return tableName;

      int lastSlash = 0;
      for(int i = fileName.length()-1; i >= 0; i--)
          if(fileName.charAt(i) == '/' || fileName.charAt(i) == '\\') {
            lastSlash = i;
            break;
          }
      tableName = fileName.substring(lastSlash+1, fileName.length() - 4);
      return tableName;
  }

  /**
   * Get the value of the column at the specified index.
   *
   * @param  columnIndex  Description of Parameter
   * @return              The column value
   * @since
   */

  public String getColumn(int columnIndex)
  {
    return columns[columnIndex];
  }

  /**
   * Get value from column at specified name.
   * If the column name is not found, throw an error.
   *
   * @param  columnName     Description of Parameter
   * @return                The column value
   * @exception  SQLException  Description of Exception
   * @since
   */

  public String getColumn(String columnName) throws SQLException
  {
    for (int loop = 0; loop < columnNames.length; loop++)
    {
      if (columnName.equalsIgnoreCase(columnNames[loop]) || columnName.equalsIgnoreCase(getTableName() + "." + columnNames[loop]))
      {
        return getColumn(loop);
      }
    }
    throw new SQLException("Column '" + columnName + "' not found.");
  }


  /**
   *Description of the Method
   *
   * @return                Description of the Returned Value
   * @exception  SQLException  Description of Exception
   * @since
   */
  public boolean next() throws SQLException, IOException {
    columns = new String[columnNames.length];
    String dataLine = null;
    current = input.getFilePointer();
    
    try {
        if (buf != null) {
          // The buffer is not empty yet, so use this first.
          dataLine = buf;
          buf = null;
        } else {
          // read new line of data from input.
          dataLine = this.input.readLine();
        }
        if (dataLine == null) {
          this.input.close();
          return false;
        }
    } catch (IOException e) {
        throw new SQLException(e.toString());
    }
    columns = parseCsvLine(dataLine);
    endLine = input.getFilePointer();
    return true;
  }


  /**
   *Description of the Method
   *
   * @since
   */
  public void close()
  {
    try
    {
      this.input.close();
      buf = null;
    }
    catch (Exception e)
    {
    }
  }


  // This code updated with code by Stuart Mottram to handle line breaks in fields
  // see bug #492063
  protected String[] parseCsvLine(String line) throws SQLException
  {
    Vector values = new Vector();
    boolean inQuotedString = false;
    String value = "";
    String orgLine = line;
    int currentPos = 0;
    int fullLine = 0;

    while (fullLine == 0){
        currentPos = 0;
        line += separator;
        while (currentPos < line.length())
            {
                char currentChar = line.charAt(currentPos);
                if (value.length() == 0 && currentChar == '"' && !inQuotedString)
                    {
                        currentPos++;
                        inQuotedString = true;
                        continue;
                    }
                if (currentChar == '"')
                    {
                        char nextChar = line.charAt(currentPos + 1);
                        if (nextChar == '"')
                            {
                                value += currentChar;
                                currentPos++;
                            }
                        else
                            {
                                if (!inQuotedString)
                                    {
                                        throw new SQLException("Unexpected '\"' in position " + currentPos + ". Line=" + orgLine);
                                    }
                                if (inQuotedString && nextChar != separator)
                                    {
                                        throw new SQLException("Expecting " + separator + " in position " + (currentPos + 1) + ". Line=" + orgLine);
                                    }
                                values.add(value);
                                value = "";
                                inQuotedString = false;
                                currentPos++;
                            }
                    }
                else
                    {
                        if (currentChar == separator)
                            {
                                if (inQuotedString)
                                    {
                                        value += currentChar;
                                    }
                                else
                                    {
                                        values.add(value);
                                        value = "";
                                    }
                            }
                        else
                            {
                                value += currentChar;
                            }
                    }
                currentPos++;
            }
        if (inQuotedString){
            // Remove extra , added at start
            value = value.substring(0,value.length()-1);
            try {
                line = this.input.readLine();
            } catch (IOException e) {
                throw new SQLException(e.toString());
            }
        } else {
            fullLine = 1;
        }

    }
    String[] retVal = new String[values.size()];
    values.copyInto(retVal);
    return retVal;
  }
  
	protected boolean newLine(String[] colNames, String[] colValues) throws IOException {
		String newLine="";
		for(int i=0;i<columnNames.length; i++) {
			boolean find = false;
			for(int j=0;j<colNames.length; j++) {
				if(colNames[j].equalsIgnoreCase(columnNames[i])) {
					if(colValues[j]==null || colValues[j].equals("null"))
						newLine=newLine+",";
					else	{
						if(colValues[j].indexOf("'")!=-1)
							colValues[j]=colValues[j].trim().substring(1,colValues[j].trim().length()-1);
						newLine=newLine+"\""+colValues[j]+"\""+",";
					}	
				find = true;	
				}		
			}
			if(!find)
				newLine=newLine+",";
		}
		if(!newLine.equals(""))
			newLine=newLine.substring(0,newLine.length()-1);			
			
      RandomAccessFile raf = new RandomAccessFile(this.fileName,"rw");
      long l = raf.length();
      raf.seek(l);
      raf.writeBytes("\n"+newLine);
      raf.close();
			
		return true;
   }

	protected boolean createTable(String[] colNames, String table) throws IOException {
		String newLine="";
		for(int i=0;i<colNames.length; i++) {
			newLine=newLine+"\""+colNames[i]+"\",";
		}	
		if(!newLine.equals(""))
			newLine=newLine.substring(0,newLine.length()-1);			

	    String fileName = tableName;

   	 RandomAccessFile raf = new RandomAccessFile(this.fileName,"rw");
      raf.writeBytes(newLine);
      raf.close();
			
		return true;
   }


	protected boolean updateField(String[] colNames, String[] colValues, String[] colWhereNames, String[] colWhereValues) throws IOException, SQLException {
	while(next()) {
  		counter++;
		boolean find = false;
		out:
		for(int i=0; i<colWhereNames.length; i++) {
			if(colWhereValues[i]!=null) {
				if(colWhereValues[i].indexOf("'")!=-1)
					colWhereValues[i]=colWhereValues[i].trim().substring(1,colWhereValues[i].trim().length()-1);				
			}		
			if(!getColumn(colWhereNames[i]).equals(colWhereValues[i])) 
				break out;
			if(i==(colWhereNames.length-1))	
				find = true;
		}		
	 
	 if(find) {
	 	for(int i=0; i< columnNames.length; i++) {
	 		for(int j=0; j<colNames.length; j++) {
	 			if(colNames[j].equalsIgnoreCase(columnNames[i]))
	 				columns[i]=colValues[j];
	 		}		
	 	}		
	 	String updatedLine = "";
	 	for(int i=0; i<columns.length; i++) {
	 		if(columns[i]==null || columns[i].equalsIgnoreCase("null")|| columns[i].equals(""))
	 			updatedLine=updatedLine+",";
	 		else {
	 			if((columns[i].indexOf("'")!=-1) ||(columns[i].indexOf("\"")!=-1))
					columns[i]=columns[i].trim().substring(1,columns[i].trim().length()-1);
	 			updatedLine=updatedLine+"\""+columns[i]+"\""+",";
	 		}	
	 		
	 	}
	 	if(!updatedLine.equals("")) {
	 		input.seek(endLine);
	 		String line="";
	 		String newLine="";
	 		
	 		while((newLine=input.readLine())!=null){
	 			line+=newLine+"\n";
	 		}	
	 		if(line!=null) {
	 			if(!line.equals(""))
	 				line=line.substring(0,line.length()-1);
	 		}		
			updatedLine=updatedLine.substring(0,updatedLine.length()-1);			
	      input.seek(current);
   	   input.writeBytes(updatedLine);
   	   if(input.getFilePointer()!=input.length()) {
//   	   	input.seek(endLine+1);		
   	   	input.writeBytes("\n");
   	   	if(line!=null){
   	   		if(!line.equals(""))
   	   		input.writeBytes(line);
   	   		input.setLength(input.getFilePointer());
   	   	}
//     	   	input.seek(endLine);			
   	   }	
	   //input.seek(endLine+1);		
//      	input.close();
		}
	  }	 
	 }
		return true;
	}
  
}

